<?php
/************************************************************************
 * This file is part of EspoCRM.
 *
 * EspoCRM - Open Source CRM application.
 * Copyright (C) 2014-2021 Yurii Kuznietsov, Taras Machyshyn, Oleksii Avramenko
 * Website: https://www.espocrm.com
 *
 * EspoCRM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EspoCRM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EspoCRM. If not, see http://www.gnu.org/licenses/.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU General Public License version 3,
 * these Appropriate Legal Notices must retain the display of the "EspoCRM" word.
 ************************************************************************/

namespace Espo\Core\FileStorage\Storages;

use Espo\Core\{
    Utils\Config,
    FileStorage\Storage,
    FileStorage\Attachment,
};

use Psr\Http\Message\StreamInterface;

use AsyncAws\S3\S3Client;

use League\Flysystem\{
    AsyncAwsS3\AsyncAwsS3Adapter,
    Filesystem,
};

use GuzzleHttp\Psr7\Stream;

use RuntimeException;

class AwsS3 implements Storage
{
    protected $filesystem;

    public function __construct(Config $config)
    {
        $bucketName = $config->get('awsS3Storage.bucketName') ?? null;
        $path = $config->get('awsS3Storage.path') ?? null;

        $region = $config->get('awsS3Storage.region') ?? null;
        $credentials = $config->get('awsS3Storage.credentials') ?? null;

        if (!$bucketName) {
            throw new RuntimeException("AWS S3 bucket name is not specified in config.");
        }

        $clientOptions = [
            'region' => $region,
        ];

        if ($credentials) {
            $clientOptions['accessKeyId'] = $credentials['key'] ?? null;
            $clientOptions['accessKeySecret'] = $credentials['secret'] ?? null;
        }

        $client = new S3Client($clientOptions);

        $adapter = new AsyncAwsS3Adapter($client, $bucketName, $path);

        $this->filesystem = new Filesystem($adapter);
    }

    public function unlink(Attachment $attachment): void
    {
        $this->filesystem->delete($attachment->getSourceId());
    }

    public function exists(Attachment $attachment): bool
    {
        return $this->filesystem->fileExists($attachment->getSourceId());
    }

    public function getSize(Attachment $attachment): int
    {
        return $this->filesystem->fileSize($attachment->getSourceId());
    }

    public function getStream(Attachment $attachment): StreamInterface
    {
        $resource = $this->filesystem->readStream($attachment->getSourceId());

        return new Stream($resource);
    }

    public function putStream(Attachment $attachment, StreamInterface $stream): void
    {
        // League\Flysystem does not support StreamInterface.
        // Need to pass a resource.

        $resource = fopen('php://temp', 'r+');

        fwrite($resource, $stream->getContents());
        rewind($resource);

        $this->filesystem->writeStream($attachment->getSourceId(), $resource);

        fclose($resource);
    }
}
